<?php

App::uses('DpdCSVExport', 'EvDpdExport.Lib');

/**
 * Export Controller
 *
 * Adds the ability to export your paid orders as a CSV for DPD
 *
 * @package       EvDPDExport.Controller
 */
class ExportController extends AppController {

	public $uses = array();

	public $adminActions = array(
		'admin_export'
	);

	public function admin_export() {
		// Get the orders to export
		$orders = $this->_getExportOrders();

		// Go through the orders and build up CSV export format
		$formattedOrders = [];
		foreach ($orders as $order) {
			$orderId = $order['Order']['id'];

			$address = $this->_decodeAddress($order['Order']['delivery_address']);

			// Address
			$formattedOrders[$orderId]['Address Line 1'] = $address['address_line_1'];
			$formattedOrders[$orderId]['Address Line 2'] = $address['address_line_2'];
			$formattedOrders[$orderId]['Address Line 3'] = $address['address_line_3'];
			$formattedOrders[$orderId]['Postcode'] = $address['postcode'];
			$formattedOrders[$orderId]['Country Code'] = $address['country_code'];

			// Contact information
			$formattedOrders[$orderId]['Contact Name'] = $order['Order']['name'];
			$formattedOrders[$orderId]['Contact Telephone'] = $order['Order']['phone'];

			// Contents Description
			$formattedOrders[$orderId]['Content Description'] = $this->_getContentsDescription($order);

			// Order Reference and nofication
			$formattedOrders[$orderId]['Customer Ref 1'] = '#' . $orderId;
			$formattedOrders[$orderId]['Notification Email'] = $order['Order']['email'];
			// If a valid UK phone number add to the notification column
			if (preg_match("/^(\+44\s?7\d{3}|\(?07\d{3}\)?)\s?\d{3}\s?\d{3}$/", $order['Order']['phone'])) {
				$formattedOrders[$orderId]['Notification SMS'] = $order['Order']['phone'];
			}
		}

		$data = DpdCSVExport::generateCSVArray($formattedOrders);

		$_serialize = 'data';
		$this->viewClass = 'CsvView.Csv';
		$this->set(compact('data', '_serialize'));
	}

/**
 * Takes the String address and breaks it into an array
 * @return Returns an annotated array of the address
 */
	protected function _decodeAddress($address) {
		$formattedAddress = [];

		$address = explode("<br>", $address);

		// Remove blank line on the end
		array_pop($address);
		$formattedAddress['country'] = array_pop($address);
		$formattedAddress['postcode'] = array_pop($address);
		$formattedAddress['address_line_1'] = array_shift($address);
		$formattedAddress['address_line_2'] = array_shift($address);
		$formattedAddress['address_line_3'] = implode(", ", $address);

		$Country = EvClassRegistry::init('EvCountry.Country');
		$country = $Country->findByName($formattedAddress['country']);
		$formattedAddress['country_code'] = $country['Country']['iso2'];

		return $formattedAddress;
	}

/**
 * Gets the orders that need to be exported
 * @return array Array of orders that match the criteria, by default PAID status
 */
	protected function _getExportOrders() {
		$Order = EvClassRegistry::init('EvCheckout.Order');
		$OrderStatus = EvClassRegistry::init('EvCheckout.OrderStatus');

		$paidStatusId = $OrderStatus->getConstant('PAID');

		$orders = $Order->find('all', [
			'conditions' => [
				'Order.order_status_id' => $paidStatusId
			]
		]);
		return $orders;
	}

/**
 * Gets the contents description for a given order
 * Unless overrridden this uses the config variable from EvDpdExport.genericContentDescription
 * @return string Gets a description of the contents
 */
	protected function _getContentsDescription($order) {
		return Configure::read('EvDpdExport.genericContentDescription');
	}
}
